//
// WebEditor
//
// Copyright 1998, François Jouen
// Based on TextEditor 
// Written by Eric Shepherd
//

#include <Application.h>
#include <Messenger.h>
#include <Message.h>
#include <Alert.h>
#include <Roster.h>
#include <Window.h>
#include <View.h>
#include <MenuBar.h>
#include <Menu.h>
#include <MenuField.h>
#include <Entry.h>
#include <Path.h>
#include <Box.h>
#include <MenuItem.h>
#include <TextView.h>
#include <FilePanel.h>
#include <ScrollView.h>
#include <OutlineListView.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "constants.h"
#include "WEApp.h"
#include "WEWindow.h"

// Constructs the window we'll be drawing into.
//
WEWindow::WEWindow(BRect frame)
			: BWindow(frame, " ", B_DOCUMENT_WINDOW, 0) {
	_InitWindow();
	Show();
}


// Create a window from a file.
//
WEWindow::WEWindow(BRect frame, entry_ref *ref)
			: BWindow(frame, "Untitled ", B_DOCUMENT_WINDOW, 0) {
	BFile file;
	
	_InitWindow();
	
	// Open the file
	
	if (file.SetTo(ref, B_READ_ONLY) == B_OK) {
		off_t length;
		char *text;
		file.GetSize(&length);		// Get the file length;
		text = (char *) malloc(length);
		if (text && (file.Read(text, length) >= B_OK)) {
			savemessage = new BMessage(B_SAVE_REQUESTED);
			if (savemessage) {
				BEntry entry(ref, true);
				BEntry parent;
				entry_ref parent_ref;
				char name[B_FILE_NAME_LENGTH];
				
				entry.GetParent(&parent);		// Get parent directory
				entry.GetName(name);
				parent.GetRef(&parent_ref);
				savemessage->AddRef("directory", &parent_ref);
				savemessage->AddString("name", name);
				textview->SetText(text, length);
				SetTitle(name);
			}
			free(text);
		}
	}	
	Show();
}


void WEWindow:: BasicHtml(void) {
//init basic HTML TAGS
	
	textview->Insert("<!-- Basic Document Tags -->\n",NULL);
	textview->Insert("<HTML>\n",NULL);
	textview->Insert("\t<HEAD>\n",NULL);
	textview->Insert("\t\t<TITLE> Welcome </TITLE>\n",NULL);
	textview->Insert("\t</HEAD>\n",NULL);
	textview->Insert("\t<BODY BACKGROUND=\"Images/Lightpin.gif\" TEXT=\"#000000\"",NULL); 
	textview->Insert("LINK=\"#ff0000\" VLINK=\"#008000\" ALINK=\"#808080\"",NULL); 
	textview->Insert("BGCOLOR=\"#c0c0c0\"\n",NULL);
	textview->Insert("\t<CENTER>",NULL);
	textview->Insert("\t<H1>",NULL);
	textview->Insert("Welcome to WebEditor 1.0!",NULL);
	textview->Insert("</H1>\n",NULL);
	textview->Insert("\t<BR>\n",NULL);
	textview->Insert("\t<BR>\n",NULL);
	textview->Insert("\t WebEditor 1.0 was conceived to help you to generate HTML 4.0 Documents \n",NULL);
	textview->Insert("\t<BR>\n",NULL);
	textview->Insert("\t<BR>\n",NULL);
	textview->Insert("\t<BR>\n",NULL);
	textview->Insert("\t<HR SIZE=3>\n",NULL);
	textview->Insert("\t Generated by WebEditor 1.0 for",NULL);
	textview->Insert("<FONT COLOR=\"#0000ff\"> B",NULL);
	textview->Insert("<FONT COLOR=\"#ff0000\">e",NULL);
	textview->Insert("<FONT COLOR=\"#000000\">OS\n",NULL);
	textview->Insert("\t<BR>\n",NULL); 
	textview->Insert("<IMG SRC=\"Images/netpositiveicon.gif\" alt=*** ALIGN=middle>\n",NULL);
	textview->Insert("\t<BR>\n",NULL);
	textview->Insert("\t&#169 Fran&ccedil;ois Jouen 1998-1999 \n",NULL);
	textview->Insert("\t</BODY>\n",NULL);
	textview->Insert("</HTML>\n",NULL);
	
}


//
// TextWindow::_InitWindow
//
// Initialize the window.
//
void WEWindow::_InitWindow(void) {
	BRect r;
	BMenu *menu;
	BMenuItem *item;

	 
	// Initialize variables
	
	savemessage = NULL;			// No saved path yet
	r = Bounds();
	BRect rtool = Bounds();
	BRect textframe = Bounds();
	
	
	// Add the menu bar
	
	
	menubar = new BMenuBar(r, "menu_bar");

	// Add File menu to menu bar
	
	menu = new BMenu("File");
	menu->AddItem(new BMenuItem("New", new BMessage(MENU_FILE_NEW), 'N'));
	
	menu->AddItem(item=new BMenuItem("Open" B_UTF8_ELLIPSIS,
					new BMessage(MENU_FILE_OPEN), 'O'));
	item->SetTarget(be_app);
	menu->AddItem(new BMenuItem("Close", new BMessage(MENU_FILE_CLOSE), 'W'));
	menu->AddSeparatorItem();
	menu->AddItem(saveitem=new BMenuItem("Save", new BMessage(MENU_FILE_SAVE), 'S'));
	saveitem->SetEnabled(false);
	menu->AddItem(new BMenuItem("Save as" B_UTF8_ELLIPSIS,
					new BMessage(MENU_FILE_SAVEAS)));
					
	//menu->AddSeparatorItem();
	//menu->AddItem(item=new BMenuItem("Page Setup" B_UTF8_ELLIPSIS,
	//				new BMessage(MENU_FILE_PAGESETUP)));
	//item->SetEnabled(false);
	//menu->AddItem(item=new BMenuItem("Print" B_UTF8_ELLIPSIS,
	//				new BMessage(MENU_FILE_PRINT), 'P'));
	//item->SetEnabled(false);
	menu->AddSeparatorItem();
	menu->AddItem(new BMenuItem("Quit", new BMessage(MENU_FILE_QUIT), 'Q'));
	menubar->AddItem(menu);
	
	// Attach the menu bar to the window
	AddChild(menubar);
	
    rtool.right = r.left+135+B_V_SCROLL_BAR_WIDTH; 
    rtool.top = menubar->Bounds().bottom+2;
    BBox* toolbar = new BBox(rtool,
			"Html",B_FOLLOW_LEFT|B_FOLLOW_TOP_BOTTOM, B_WILL_DRAW | B_FRAME_EVENTS, B_FANCY_BORDER);
	toolbar->SetLabel("HTML Elements");
	AddChild(toolbar);
	
	r.top = rtool.top+5;
	r.left +=5;
	r.right =rtool.right-B_V_SCROLL_BAR_WIDTH-5;
	r.bottom =rtool.bottom-B_H_SCROLL_BAR_HEIGHT*3;
	
	
	outline = new BOutlineListView(r, "Html", 
    			B_SINGLE_SELECTION_LIST, B_FOLLOW_ALL,
    			 B_WILL_DRAW | B_NAVIGABLE);
    
    outline->SetInvocationMessage(new BMessage(OUTLINE_REQUESTED));
   	 
    outline->AddItem(Elements = new BStringItem("Document Elements"));
    outline->AddUnder(Tags = new BStringItem("Spot Id"), Elements);
    outline->AddUnder(Tags = new BStringItem("Range"), Elements);
    outline->AddUnder(Tags = new BStringItem("Span"), Elements);
    outline->AddUnder(Tags = new BStringItem("Division"), Elements);
    outline->AddUnder(Tags = new BStringItem("Banner"), Elements);
    outline->AddUnder(Tags = new BStringItem("Body"), Elements);
    outline->AddUnder(Tags = new BStringItem("Title"), Elements);
    outline->AddUnder(Tags = new BStringItem("Next Id"), Elements);
    outline->AddUnder(Tags = new BStringItem("Meta"), Elements);
    outline->AddUnder(Tags = new BStringItem("Link"), Elements);
    outline->AddUnder(Tags = new BStringItem("Is Index"), Elements);
    outline->AddUnder(Tags = new BStringItem("Base"), Elements);
    outline->AddUnder(Tags = new BStringItem("Head"), Elements);
    outline->AddUnder(Tags = new BStringItem("Html"), Elements);
    outline->AddUnder(Tags = new BStringItem("Basic Structure"), Elements);
    outline->Collapse(Elements); 
    
    outline->AddItem(Elements = new BStringItem("Block Elements"));
    outline->AddUnder(Tags = new BStringItem("Word Break"), Elements);
    outline->AddUnder(Tags = new BStringItem("Tab"), Elements);
    outline->AddUnder(Tags = new BStringItem("Preformatted Text"), Elements);
    outline->AddUnder(Tags = new BStringItem("Plain Text"), Elements);
    outline->AddUnder(Tags = new BStringItem("Paragraph"), Elements);
    outline->AddUnder(Tags = new BStringItem("No Break"), Elements);
    outline->AddUnder(Tags = new BStringItem("Columns"), Elements);
    outline->AddUnder(Tags = new BStringItem("Line Break"), Elements);
    outline->AddUnder(Tags = new BStringItem("Inline Quote"), Elements);
    outline->AddUnder(Tags = new BStringItem("Horizontal Rule"), Elements);
    outline->AddUnder(Tags = new BStringItem("Headings"), Elements);
    outline->AddUnder(new BStringItem("H6"), Tags); 
    outline->AddUnder(new BStringItem("H5"), Tags); 
    outline->AddUnder(new BStringItem("H4"), Tags); 
    outline->AddUnder(new BStringItem("H3"), Tags); 
    outline->AddUnder(new BStringItem("H2"), Tags); 
    outline->AddUnder(new BStringItem("H1"), Tags); 
    outline->Collapse(Tags); 
    outline->AddUnder(Tags = new BStringItem("Font Size"), Elements);
    outline->AddUnder(new BStringItem("Size 7"), Tags); 
    outline->AddUnder(new BStringItem("Size 6"), Tags); 
    outline->AddUnder(new BStringItem("Size 5"), Tags); 
    outline->AddUnder(new BStringItem("Size 4"), Tags); 
    outline->AddUnder(new BStringItem("Size 3"), Tags); 
    outline->AddUnder(new BStringItem("Size 2"), Tags); 
    outline->AddUnder(new BStringItem("Size 1"), Tags); 
    outline->Collapse(Tags); 
    outline->AddUnder(Tags = new BStringItem("Font Color"), Elements);
    outline->AddUnder(Tags = new BStringItem("Centered"), Elements);
    outline->AddUnder(Tags = new BStringItem("By Line"), Elements);
    outline->AddUnder(Tags = new BStringItem("Block Quote"), Elements);
    outline->AddUnder(Tags = new BStringItem("Base Font"), Elements);
    outline->AddUnder(Tags = new BStringItem("Aligment"), Elements);
    outline->AddUnder(new BStringItem("Left"), Tags); 
    outline->AddUnder(new BStringItem("Right"), Tags);
    outline->AddUnder(new BStringItem("Center"), Tags);
    outline->AddUnder(new BStringItem("Justified"), Tags);
    outline->Collapse(Tags); 
    outline->Collapse(Elements); 
    
    outline->AddItem(Elements = new BStringItem("Characters Elements"));
    outline->AddUnder(Tags = new BStringItem("Underlined"), Elements);
    outline->AddUnder(Tags = new BStringItem("Typewriter"), Elements);
    outline->AddUnder(Tags = new BStringItem("Superscript"), Elements);
    outline->AddUnder(Tags = new BStringItem("Subscript"), Elements);
    outline->AddUnder(Tags = new BStringItem("Strong Emphasis"), Elements);
    outline->AddUnder(Tags = new BStringItem("Strikethru"), Elements);
    outline->AddUnder(Tags = new BStringItem("Small"), Elements);
    outline->AddUnder(Tags = new BStringItem("Keyboard"), Elements);
    outline->AddUnder(Tags = new BStringItem("Italic"), Elements);
    outline->AddUnder(Tags = new BStringItem("Emphasis"), Elements);  
    outline->AddUnder(Tags = new BStringItem("Bold"), Elements);  
    outline->AddUnder(Tags = new BStringItem("Blink"), Elements);
    outline->AddUnder(Tags = new BStringItem("Big"), Elements);
    outline->Collapse(Elements);
        
    outline->AddItem(Elements = new BStringItem("Information Elements"));
    outline->AddUnder(Tags = new BStringItem("Variable"), Elements);
    outline->AddUnder(Tags = new BStringItem("Sample"), Elements);
    outline->AddUnder(Tags = new BStringItem("Person"), Elements);
    outline->AddUnder(Tags = new BStringItem("Note"), Elements);
    outline->AddUnder(Tags = new BStringItem("Insert"), Elements);
    outline->AddUnder(Tags = new BStringItem("Footnote"), Elements);
    outline->AddUnder(Tags = new BStringItem("Example"), Elements);
    outline->AddUnder(Tags = new BStringItem("Delete"), Elements);
    outline->AddUnder(Tags = new BStringItem("Definition"), Elements);
    outline->AddUnder(Tags = new BStringItem("Comment"), Elements);
    outline->AddUnder(Tags = new BStringItem("Code"), Elements);
    outline->AddUnder(Tags = new BStringItem("Citation"), Elements);
    outline->AddUnder(Tags = new BStringItem("Author"), Elements);
    outline->AddUnder(Tags = new BStringItem("Argument"), Elements);
    outline->AddUnder(Tags = new BStringItem("Address"), Elements);
    outline->AddUnder(Tags = new BStringItem("Acronym"), Elements);
    outline->AddUnder(Tags = new BStringItem("Abbreviation"), Elements);
    outline->Collapse(Elements);
    
    outline->AddItem(Elements = new BStringItem("Form Elements"));
    outline->AddUnder(Tags = new BStringItem("Text Area"), Elements);
    outline->AddUnder(Tags = new BStringItem("Select"), Elements);
    outline->AddUnder(Tags = new BStringItem("Option"), Elements);
    outline->AddUnder(Tags = new BStringItem("Input"), Elements);
    outline->AddUnder(Tags = new BStringItem("Label"), Elements);
    outline->AddUnder(Tags = new BStringItem("Button"), Elements);
    outline->AddUnder(Tags = new BStringItem("Fieldset"), Elements);
    outline->AddUnder(Tags = new BStringItem("Legend"), Elements);
    outline->AddUnder(Tags = new BStringItem("Form"), Elements);
    outline->Collapse(Elements);
        
    outline->AddItem(Elements = new BStringItem("Frame Elements"));
    outline->AddUnder(Tags = new BStringItem("No Frames"), Elements);
    outline->AddUnder(Tags = new BStringItem("Frame"), Elements);
    outline->AddUnder(Tags = new BStringItem("Frame Set"), Elements);
    outline->Collapse(Elements);
    
    outline->AddItem(Elements = new BStringItem("List Elements"));
    outline->AddUnder(Tags = new BStringItem("Definition Data"), Elements);
    outline->AddUnder(Tags = new BStringItem("Definition Item"), Elements);
    outline->AddUnder(Tags = new BStringItem("List Item"), Elements);
    outline->AddUnder(Tags = new BStringItem("Definition List"), Elements);
    outline->AddUnder(Tags = new BStringItem("Menu List"), Elements);
    outline->AddUnder(Tags = new BStringItem("Directory List"), Elements);  
    outline->AddUnder(Tags = new BStringItem("Ordered List"), Elements);
    outline->AddUnder(Tags = new BStringItem("Unordered List"), Elements);
    outline->Collapse(Elements);
    
    outline->AddItem(Elements = new BStringItem("Table Elements"));
    outline->AddUnder(Tags = new BStringItem("Table Data"), Elements);
    outline->AddUnder(Tags = new BStringItem("Table Foot"), Elements);
    outline->AddUnder(Tags = new BStringItem("Table Body"), Elements);
    outline->AddUnder(Tags = new BStringItem("Table Head"), Elements);
    outline->AddUnder(Tags = new BStringItem("Table Row"), Elements);
    outline->AddUnder(Tags = new BStringItem("Table Column"), Elements);
    outline->AddUnder(Tags = new BStringItem("Table ColGroup"), Elements);
    outline->AddUnder(Tags = new BStringItem("Table Caption"), Elements);
    outline->AddUnder(Tags = new BStringItem("Table"), Elements);
    outline->Collapse(Elements);
    
    outline->AddItem(Elements = new BStringItem("Links Elements"));
    outline->AddUnder(Tags = new BStringItem("Mail To"), Elements);
    outline->AddUnder(Tags = new BStringItem("Hyperlink"), Elements);
    outline->AddUnder(Tags = new BStringItem("Bookmark"), Elements);
    outline->Collapse(Elements);
    
    outline->AddItem(Elements = new BStringItem("Objects Elements"));
    outline->AddUnder(Tags = new BStringItem("Generic Object"), Elements);
    outline->AddUnder(Tags = new BStringItem("Embedded Object"), Elements);
    outline->AddUnder(Tags = new BStringItem("Background Sound"), Elements);
    outline->AddUnder(Tags = new BStringItem("Marquee"), Elements);
    outline->AddUnder(Tags = new BStringItem("Inline Figure"), Elements);
    outline->AddUnder(Tags = new BStringItem("Client Side Image Map"), Elements);
    outline->AddUnder(new BStringItem("Map Area"), Tags); 
    outline->AddUnder(new BStringItem("Map Name"), Tags);
    outline->Collapse(Tags);
    outline->AddUnder(Tags = new BStringItem("Inline Image"), Elements);
    
    outline->Collapse(Elements);
	
	
	toolbar->AddChild(new BScrollView("Html", outline,B_FOLLOW_ALL, 
	                                  0, false, true,B_NO_BORDER));
	
	
	// Add the text view
	textframe.left +=rtool.right+5;
	textframe.top = menubar->Bounds().bottom+2;
	textframe.right -= B_V_SCROLL_BAR_WIDTH;
	textframe.bottom -= B_H_SCROLL_BAR_HEIGHT;
	
	BRect textrect = textframe;
	textrect.OffsetTo(B_ORIGIN);
	r.InsetBy(3.0,3.0);
	textview = new BTextView(textframe, "text_view", textrect,
				B_FOLLOW_ALL_SIDES, B_WILL_DRAW|B_PULSE_NEEDED);
	AddChild(scrollview = new BScrollView("scroll_view", textview,
				B_FOLLOW_ALL_SIDES, 0, true, true, B_NO_BORDER));
	textview->SetDoesUndo(true);
	
	textview->MakeFocus(true);
	
	
	// Add the Edit menu to the menu bar

	menu = new BMenu("Edit");
	menu->AddItem(item=new BMenuItem("Undo", new BMessage(B_UNDO), 'Z'));
	item->SetTarget(textview);
	menu->AddSeparatorItem();
	menu->AddItem(item=new BMenuItem("Cut", new BMessage(B_CUT), 'X'));
	item->SetTarget(textview);
	menu->AddItem(item=new BMenuItem("Copy", new BMessage(B_COPY), 'C'));
	item->SetTarget(textview);
	menu->AddItem(item=new BMenuItem("Paste", new BMessage(B_PASTE), 'V'));
	item->SetTarget(textview);
	menu->AddSeparatorItem();
	menu->AddItem(item=new BMenuItem("Select All", new BMessage(B_SELECT_ALL), 'A'));
	item->SetTarget(textview);
	
	menu->AddItem(new BMenuItem("Delete All",new BMessage(MENU_EDIT_CLEAR),'D'));
	item->SetTarget(textview);
	menubar->AddItem(menu);
	
	//help menu
	menu = new BMenu("Help");
	menu->AddItem(new BMenuItem("About...",new BMessage(MENU_HELP_ABOUT)));
	menubar->AddItem(menu);
	
	
	// Create the save filepanel for this window
	
	savePanel = new BFilePanel(B_SAVE_PANEL, new BMessenger(this), NULL,
				B_FILE_NODE, false);

	// Tell the application that there's one more window
	// and get the number for this untitled window.
	
	Register(true);
	Minimize(false);		// So Show() doesn't really make it visible
	
	
}


//
// WEWindow::FrameResized
//
// Adjust the size of the BTextView's text rectangle
// when the window is resized.
//
void WEWindow::FrameResized(float width, float height) {
	BRect textrect = textview->TextRect();
	
	textrect.right = textrect.left + (width - B_V_SCROLL_BAR_WIDTH - 3.0);
	textview->SetTextRect(textrect);
}


//
// WEWindow::~WEWindow
//
// Destruct the window.  This calls Unregister().
//
WEWindow::~WEWindow() {
	Unregister();
	if (savemessage) {
		delete savemessage;
	}
	delete savePanel;
}





//
// WEWindow::MessageReceived
//
// Called when a message is received by our
// application.
//




void WEWindow::MessageReceived(BMessage *message) {

	switch(message->what) {
	case OUTLINE_REQUESTED:
	{
		switch(outline->FullListCurrentSelection())
		{ 
		// Document elements
		case 1:BasicHtml();
		break;
		case 2:textview->Insert(TAG_STR_2,NULL);
		break;
		case 3:textview->Insert(TAG_STR_3,NULL);
		break;
		case 4:textview->Insert(TAG_STR_4,NULL);
		break;
		case 5:textview->Insert(TAG_STR_5,NULL);
		break;
		case 6:{
		textview->Insert(TAG_STR_6,NULL);
		textview->Insert(TAG_STR_6_1,NULL);
		textview->Insert(TAG_STR_6_2,NULL);
		textview->Insert(TAG_STR_6_3,NULL);
		textview->Insert(TAG_STR_6_4,NULL);
		}
		break;
		case 7:textview->Insert(TAG_STR_7,NULL);
		break;
		case 8:textview->Insert(TAG_STR_8,NULL);
		break;
		case 9: textview->Insert(TAG_STR_9,NULL);
		break;
		// Block elements
		case 10: textview->Insert(TAG_STR_10,NULL);
		break;
		case 11: textview->Insert(TAG_STR_11,NULL);
		break;
		case 12: textview->Insert(TAG_STR_12,NULL);
		break;
		case 13: textview->Insert(TAG_STR_13,NULL);
		break;
		case 14: textview->Insert(TAG_STR_14,NULL);
		break;
		case 15: textview->Insert(TAG_STR_15,NULL);
		break;
		//block elements
		case 18: textview->Insert(TAG_STR_18,NULL);
		break;
		case 19: textview->Insert(TAG_STR_19,NULL);
		break;
		case 20: textview->Insert(TAG_STR_20,NULL);
		break;
		case 21: textview->Insert(TAG_STR_21,NULL);
		break;
		case 22: textview->Insert(TAG_STR_22,NULL);
		break;
		case 23: textview->Insert(TAG_STR_23,NULL);
		break;
		case 24: textview->Insert(TAG_STR_24,NULL);
		break;
		case 25: textview->Insert(TAG_STR_25,NULL);
		break;
		case 26: textview->Insert(TAG_STR_26,NULL);
		break;
		case 28: textview->Insert(TAG_STR_28,NULL);
		break;
		case 29: textview->Insert(TAG_STR_29,NULL);
		break;
		case 30: textview->Insert(TAG_STR_30,NULL);
		break;
		case 31: textview->Insert(TAG_STR_31,NULL);
		break;
		case 32: textview->Insert(TAG_STR_32,NULL);
		break;
		case 33: textview->Insert(TAG_STR_33,NULL);
		break;
		case 34: textview->Insert(TAG_STR_34,NULL);
		break;
		
		case 36: textview->Insert(TAG_STR_36,NULL);
		break;
		case 37: textview->Insert(TAG_STR_37,NULL);
		break;
		case 38: textview->Insert(TAG_STR_38,NULL);
		break;
		case 39: textview->Insert(TAG_STR_39,NULL);
		break;
		case 40: textview->Insert(TAG_STR_40,NULL);
		break;
		case 41: textview->Insert(TAG_STR_41,NULL);
		break;
		case 42: textview->Insert(TAG_STR_42,NULL);
		break;
		case 43: textview->Insert(TAG_STR_43,NULL);
		break;
		case 44: textview->Insert(TAG_STR_44,NULL);
		break;
		case 45: textview->Insert(TAG_STR_45,NULL);
		break;
		case 46: textview->Insert(TAG_STR_46,NULL);
		break;
		case 47: textview->Insert(TAG_STR_47,NULL);
		break;
		case 48: textview->Insert(TAG_STR_48,NULL);
		break;
		case 49: textview->Insert(TAG_STR_49,NULL);
		break;
		case 50: textview->Insert(TAG_STR_50,NULL);
		break;
		case 51: textview->Insert(TAG_STR_51,NULL);
		break;
		// character elements
		case 53: textview->Insert(TAG_STR_53,NULL);
		break;
		case 54: textview->Insert(TAG_STR_54,NULL);
		break;
		case 55: textview->Insert(TAG_STR_55,NULL);
		break;
		case 56: textview->Insert(TAG_STR_56,NULL);
		break;
		case 57: textview->Insert(TAG_STR_57,NULL);
		break;
		case 58: textview->Insert(TAG_STR_58,NULL);
		break;
		case 59: textview->Insert(TAG_STR_59,NULL);
		break;
		case 60: textview->Insert(TAG_STR_60,NULL);
		break;
		case 61: textview->Insert(TAG_STR_61,NULL);
		break;
		case 62: textview->Insert(TAG_STR_62,NULL);
		break;
		case 63: textview->Insert(TAG_STR_63,NULL);
		break;
		case 64: textview->Insert(TAG_STR_64,NULL);
		break;
		// information elements
		case 65: textview->Insert(TAG_STR_65,NULL);
		break;
		case 67: textview->Insert(TAG_STR_67,NULL);
		break;
		case 68: textview->Insert(TAG_STR_68,NULL);
		break;
		case 69: textview->Insert(TAG_STR_69,NULL);
		break;
		case 70: textview->Insert(TAG_STR_70,NULL);
		break;
		case 71: textview->Insert(TAG_STR_71,NULL);
		break;
		case 72: textview->Insert(TAG_STR_72,NULL);
		break;
		case 73: textview->Insert(TAG_STR_73,NULL);
		break;
		case 74: textview->Insert(TAG_STR_74,NULL);
		break;
		case 75: textview->Insert(TAG_STR_75,NULL);
		break;
		case 76: textview->Insert(TAG_STR_76,NULL);
		break;
		case 77: textview->Insert(TAG_STR_77,NULL);
		break;
		case 78: textview->Insert(TAG_STR_78,NULL);
		break;
		case 79: textview->Insert(TAG_STR_79,NULL);
		break;
		case 80: textview->Insert(TAG_STR_80,NULL);
		break;
		case 81: textview->Insert(TAG_STR_81,NULL);
		break;
		case 82: textview->Insert(TAG_STR_82,NULL);
		break;
		case 83: textview->Insert(TAG_STR_83,NULL);
		break;
		// form elements
		case 85: textview->Insert(TAG_STR_85,NULL);
		break;
		case 86: textview->Insert(TAG_STR_86,NULL);
		break;
		case 87: textview->Insert(TAG_STR_87,NULL);
		break;
		case 88: textview->Insert(TAG_STR_88,NULL);
		break;
		case 89: textview->Insert(TAG_STR_89,NULL);
		break;
		case 90: {
		textview->Insert(TAG_STR_90,NULL);
		textview->Insert(TAG_STR_901,NULL);
		}
		break;
		case 91: textview->Insert(TAG_STR_91,NULL);
		break;
		case 92: textview->Insert(TAG_STR_92,NULL);
		break;
		case 93: textview->Insert(TAG_STR_93,NULL);
		break;
		//frame elements
		case 95: textview->Insert(TAG_STR_95,NULL);
		break;
		case 96: textview->Insert(TAG_STR_96,NULL);
		break;
		case 97: textview->Insert(TAG_STR_97,NULL);
		break;
		//list elements
		case 99: textview->Insert(TAG_STR_99,NULL);
		break;
		case 100: textview->Insert(TAG_STR_100,NULL);
		break;
		case 101: textview->Insert(TAG_STR_101,NULL);
		break;
		case 102: textview->Insert(TAG_STR_102,NULL);
		break;
		case 103: textview->Insert(TAG_STR_103,NULL);
		break;
		case 104: textview->Insert(TAG_STR_104,NULL);
		break;
		case 105: textview->Insert(TAG_STR_105,NULL);
		break;
		case 106: textview->Insert(TAG_STR_106,NULL);
		break;
		// table elements
		case 108: textview->Insert(TAG_STR_108,NULL);
		break;
		case 109: textview->Insert(TAG_STR_109,NULL);
		break;
		case 110: textview->Insert(TAG_STR_110,NULL);
		break;
		case 111: textview->Insert(TAG_STR_111,NULL);
		break;
		case 112: textview->Insert(TAG_STR_112,NULL);
		break;
		case 113: textview->Insert(TAG_STR_113,NULL);
		break;
		case 114: textview->Insert(TAG_STR_114,NULL);
		break;
		case 115: textview->Insert(TAG_STR_115,NULL);
		break;
		case 116: textview->Insert(TAG_STR_116,NULL);
		break;
		//Basic links
		case 118: textview->Insert(TAG_STR_118,NULL);
		break;
		case 119: textview->Insert(TAG_STR_119,NULL);
		break;
		case 120: textview->Insert(TAG_STR_120,NULL);
		break;
		// Object elements
		case 122: textview->Insert(TAG_STR_122,NULL);
		break;
		case 124: textview->Insert(TAG_STR_124,NULL);
		break;
		case 125: textview->Insert(TAG_STR_125,NULL);
		break;
		case 126:{
			 textview->Insert(TAG_STR_126,NULL);
			 textview->Insert(TAG_STR_126_1,NULL);
			 textview->Insert(TAG_STR_126_2,NULL);
		}		 
		break;
		case 127: textview->Insert(TAG_STR_127,NULL);
		break;
		case 128: textview->Insert(TAG_STR_128,NULL);
		break;
		case 129: textview->Insert(TAG_STR_129,NULL);
		break;
		case 130: textview->Insert(TAG_STR_130,NULL);
		break;
		}	
	}
	break;
	
	
	
		case WINDOW_REGISTRY_ADDED:
			{
				char s[22];
				BRect rect;
				if (message->FindInt32("new_window_number", &window_id) == B_OK) {
					if (!savemessage) {		// if it's untitled
						sprintf(s, "File%ld.html", window_id);
						SetTitle(s);
					}
				}
				if (message->FindRect("rect", &rect) == B_OK) {
					MoveTo(rect.LeftTop());
					ResizeTo(rect.Width(), rect.Height());
				}
				Minimize(false);
			}
			break;
			
		case MENU_FILE_NEW:
			{
				BRect r;
				r = Frame();
				new WEWindow(r);
			}
			break;
		
		case MENU_EDIT_CLEAR:	textview->SetText("");;
				break;		
		case MENU_FILE_CLOSE:
			Quit();
			break;
		case MENU_FILE_QUIT:
			be_app->PostMessage(B_QUIT_REQUESTED);
			break;
		case MENU_FILE_SAVEAS:
			savePanel->Show();
			break;
		case MENU_FILE_SAVE:
			Save(NULL);
			break;
		case B_SAVE_REQUESTED:
			Save(message);
			break;
		case MENU_HELP_ABOUT:{
			BAlert		*alert;
			alert= new BAlert("About ", "WebEditor for BeOS™\n©François Jouen 1999.\ne-mail:jouen@epeire.univ-rouen.fr", "OK");
			alert->Go();
	
		}	
		break;				
					
		default:
			BWindow::MessageReceived(message);
			break;
	}
}


//
// WEWindow::Register
//
// Since MessageWorld can have multiple windows and
// we need to know when there aren't any left so the
// application can be shut down, this function is used
// to tell the application that a new window has been
// opened.
//
// If the need_id argument is true, we'll specify true
// for the "need_id" field in the message we send; this
// will cause the application to send back a
// WINDOW_REGISTRY_ADDED message containing the window's
// unique ID number.  If this argument is false, we won't
// request an ID.
//
void WEWindow::Register(bool need_id) {
	BMessenger messenger(APP_SIGNATURE);
	BMessage message(WINDOW_REGISTRY_ADD);
	
	message.AddBool("need_id", need_id);
	messenger.SendMessage(&message, this);
}


//
// WEWindow::Unregister
//
// Unregisters a window.  This tells the application that
// one fewer windows are open.  The application will
// automatically quit if the count goes to zero because
// of this call.
//
void WEWindow::Unregister(void) {
	BMessenger messenger(APP_SIGNATURE);
	
	messenger.SendMessage(new BMessage(WINDOW_REGISTRY_SUB));
}


//
// WEWindow::QuitRequested
//
// Here we just give permission to close the window.
//
bool WEWindow::QuitRequested() {
	return true;
}


//
// WEWindow::Save
//
// Save the contents of the window.  The message specifies
// where to save it (see BFilePanel in the Storage Kit chapter
// of the Be Book).
//
status_t WEWindow::Save(BMessage *message) {
	entry_ref ref;		// For the directory to save into
	status_t err;		// For the return code
	const char *name;	// For the filename
	BPath path;		// For the pathname
	BEntry entry;		// Used to make the path
	FILE *f;		// Standard Posix file
	
	// If a NULL is passed for the message pointer, use
	// the value we've cached; this lets us do saves without
	// thinking.
	
	if (!message) {
		message = savemessage;
		if (!message) {
			return B_ERROR;
		}
	}
	
	// Peel the entry_ref and name of the directory and
	// file out of the message.
	
	if ((err=message->FindRef("directory", &ref)) != B_OK) {
		return err;
	}
	if ((err=message->FindString("name", &name)) != B_OK) {
		return err;
	}
	
	// Take the directory and create a pathname out of it
	
	if ((err=entry.SetTo(&ref)) != B_OK) {
		return err;
	}
	entry.GetPath(&path);		// Create a pathname for the directory
	path.Append(name);			// Tack on the filename
	
	// Now we can save the file.
	
	if (!(f = fopen(path.Path(), "w"))) {
		return B_ERROR;
	}
	
	err = fwrite(textview->Text(), 1, textview->TextLength(), f);
	fclose(f);
	if (err >= 0) {
		SetTitle(name);
		saveitem->SetEnabled(true);
		if (savemessage != message) {
			if (savemessage) {
				delete savemessage;
			}
			savemessage = new BMessage(message);
		}
	}
	return err;
}
